@@ -47,9 +47,9 @@ class ExtendedAttributes:
 		else: self.attr_dict = attr_dict
 
 	def __eq__(self, ea):
-		"""Equal if all attributes and index are equal"""
+		"""Equal if all attributes are equal"""
 		assert isinstance(ea, ExtendedAttributes)
-		return ea.index == self.index and ea.attr_dict == self.attr_dict
+		return ea.attr_dict == self.attr_dict
 	def __ne__(self, ea): return not self.__eq__(ea)
 
 	def get_indexpath(self): return self.index and '/'.join(self.index) or '.'
@@ -59,6 +59,9 @@ class ExtendedAttributes:
 		try: attr_list = rp.conn.xattr.listxattr(rp.path)
 		except IOError, exc:
 			if exc[0] == errno.EOPNOTSUPP: return # if not sup, consider empty
+			if exc[0] == errno.EACCES:
+				log.Log("Warning: listattr(%s): %s" % (rp.path, exc), 3)
+				return
 			raise
 		for attr in attr_list:
 			if not attr.startswith('user.'):
@@ -74,7 +77,16 @@ class ExtendedAttributes:
 	def clear_rp(self, rp):
 		"""Delete all the extended attributes in rpath"""
 		for name in rp.conn.xattr.listxattr(rp.path):
-			rp.conn.xattr.removexattr(rp.path, name)
+			try:
+				rp.conn.xattr.removexattr(rp.path, name)
+			except IOError, exc:
+				# SELinux attributes cannot be removed, and we don't want
+				# to bail out or be too noisy at low log levels.
+				if exc[0] == errno.EACCES:
+					log.Log("Warning: unable to remove xattr %s from %s"
+						% (name, rp.path), 7)
+					continue
+				else: raise
 
 	def write_to_rp(self, rp):
 		"""Write extended attributes to rpath rp"""
@@ -216,21 +228,19 @@ class AccessControlLists:
 
 	def entrytuple_to_text(self, entrytuple):
 		"""Return text version of entrytuple, as in getfacl"""
-		type, name_pair, perms = entrytuple
-		if type == posix1e.ACL_USER_OBJ:
-			text = 'user::'
-		elif type == posix1e.ACL_USER:
+		tagchar, name_pair, perms = entrytuple
+		if tagchar == "U": text = 'user::'
+		elif tagchar == "u":
 			uid, uname = name_pair
 			text = 'user:%s:' % (uname or uid)
-		elif type == posix1e.ACL_GROUP_OBJ:
+		elif tagchar == "G":
 			text = 'group::'
-		elif type == posix1e.ACL_GROUP:
+		elif tagchar == "g":
 			gid, gname = name_pair
 			text = 'group:%s:' % (gname or gid)
-		elif type == posix1e.ACL_MASK:
-			text = 'mask::'
+		elif tagchar == "M": text = 'mask::'
 		else:
-			assert type == posix1e.ACL_OTHER, type
+			assert tagchar == "O", tagchar
 			text = 'other::'
 
 		permstring = '%s%s%s' % (perms & 4 and 'r' or '-',
@@ -239,32 +249,36 @@ class AccessControlLists:
 		return text+permstring
 
 	def text_to_entrytuple(self, text):
-		"""Return entrytuple given text like 'user:foo:r--'"""
+		"""Return entrytuple given text like 'user:foo:r--'
+
+		See the acl_to_list function for entrytuple documentation.
+
+		"""
 		typetext, qualifier, permtext = text.split(':')
 		if qualifier:
-			try: id = int(qualifier)
+			try: uid = int(qualifier)
 			except ValueError: namepair = (None, qualifier)
 			else: namepair = (uid, None)
 
-			if typetext == 'user': type = posix1e.ACL_USER
+			if typetext == 'user': typechar = "u"
 			else:
 				assert typetext == 'group', (typetext, text)
-				type = posix1e.ACL_GROUP
+				typechar = "g"
 		else:
 			namepair = None
-			if typetext == 'user': type = posix1e.ACL_USER_OBJ
-			elif typetext == 'group': type = posix1e.ACL_GROUP_OBJ
-			elif typetext == 'mask': type = posix1e.ACL_MASK
+			if typetext == 'user': typechar = "U"
+			elif typetext == 'group': typechar = "G"
+			elif typetext == 'mask': typechar = "M"
 			else:
 				assert typetext == 'other', (typetext, text)
-				type = posix1e.ACL_OTHER
+				typechar = "O"
 
 		assert len(permtext) == 3, (permtext, text)
 		read, write, execute = permtext
 		perms = ((read == 'r') << 2 |
 				 (write == 'w') << 1 |
 				 (execute == 'x'))
-		return (type, namepair, perms)
+		return (typechar, namepair, perms)
 
 	def cmp_entry_list(self, l1, l2):
 		"""True if the lists have same entries.  Assume preordered"""
@@ -292,7 +306,6 @@ class AccessControlLists:
 
 		"""
 		assert isinstance(acl, self.__class__)
-		if self.index != acl.index: return 0
 		if self.is_basic(): return acl.is_basic()
 		return (self.cmp_entry_list(self.entry_list, acl.entry_list) and
 				self.cmp_entry_list(self.default_entry_list,
@@ -302,9 +315,6 @@ class AccessControlLists:
 	
 	def eq_verbose(self, acl):
 		"""Returns same as __eq__ but print explanation if not equal"""
-		if self.index != acl.index:
-			print "index %s not equal to index %s" % (self.index, acl.index)
-			return 0
 		if not self.cmp_entry_list(self.entry_list, acl.entry_list):
 			print "ACL entries for %s compare differently" % (self.index,)
 			return 0
@@ -377,15 +387,33 @@ def acl_to_list(acl):
 	lost when moved to another system.
 
 	The result will be a list of tuples.  Each tuple will have the
-	form (acltype, (uid or gid, uname or gname) or None,
-	permissions as an int).
+	form (acltype, (uid or gid, uname or gname) or None, permissions
+	as an int).  acltype is encoded as a single character:
+
+	U - ACL_USER_OBJ
+	u - ACL_USER
+	G - ACL_GROUP_OBJ
+	g - ACL_GROUP
+	M - ACL_MASK
+	O - ACL_OTHER
 
 	"""
+	def acltag_to_char(tag):
+		if tag == posix1e.ACL_USER_OBJ: return "U"
+		elif tag == posix1e.ACL_USER: return "u"
+		elif tag == posix1e.ACL_GROUP_OBJ: return "G"
+		elif tag == posix1e.ACL_GROUP: return "g"
+		elif tag == posix1e.ACL_MASK: return "M"
+		else:
+			assert tag == posix1e.ACL_OTHER, tag
+			return "O"
+
 	def entry_to_tuple(entry):
-		if entry.tag_type == posix1e.ACL_USER:
+		tagchar = acltag_to_char(entry.tag_type)
+		if tagchar == "u":
 			uid = entry.qualifier
 			owner_pair = (uid, user_group.uid2uname(uid))
-		elif entry.tag_type == posix1e.ACL_GROUP:
+		elif tagchar == "g":
 			gid = entry.qualifier
 			owner_pair = (gid, user_group.gid2gname(gid))
 		else: owner_pair = None
@@ -393,7 +421,7 @@ def acl_to_list(acl):
 		perms = (entry.permset.read << 2 | 
 				 entry.permset.write << 1 |
 				 entry.permset.execute)
-		return (entry.tag_type, owner_pair, perms)
+		return (tagchar, owner_pair, perms)
 	return map(entry_to_tuple, acl)
 
 def list_to_acl(entry_list, map_names = 1):
@@ -403,7 +431,20 @@ def list_to_acl(entry_list, map_names = 1):
 	current system, and drop if not available.  Otherwise just use the
 	same id.
 
+	See the acl_to_list function for the format of an acllist.
+
 	"""
+	def char_to_acltag(typechar):
+		"""Given typechar, query posix1e module for appropriate constant"""
+		if typechar == "U": return posix1e.ACL_USER_OBJ
+		elif typechar == "u": return posix1e.ACL_USER
+		elif typechar == "G": return posix1e.ACL_GROUP_OBJ
+		elif typechar == "g": return posix1e.ACL_GROUP
+		elif typechar == "M": return posix1e.ACL_MASK
+		else:
+			assert typechar == "O", typechar
+			return posix1e.ACL_OTHER
+
 	def warn_drop(name):
 		"""Warn about acl with name getting dropped"""
 		global dropped_acl_names
@@ -427,23 +468,23 @@ def list_to_acl(entry_list, map_names = 1):
 			return Map.get_id_from_id(id)
 
 	acl = posix1e.ACL()
-	for tag, owner_pair, perms in entry_list:
+	for typechar, owner_pair, perms in entry_list:
 		id = None
 		if owner_pair:
 			if map_names:
-				if tag == posix1e.ACL_USER: id = map_id_name(owner_pair, 0)
+				if typechar == "u": id = map_id_name(owner_pair, 0)
 				else:
-					assert tag == posix1e.ACL_GROUP, (tag, owner_pair, perms)
+					assert typechar == "g", (typechar, owner_pair, perms)
 					id = map_id_name(owner_pair, 1)
 				if id is None:
 					warn_drop(owner_pair[1])
 					continue
 			else:
-				assert owner_pair[0] is not None, (tag, owner_pair, perms)
+				assert owner_pair[0] is not None, (typechar, owner_pair, perms)
 				id = owner_pair[0]
 
 		entry = posix1e.Entry(acl)
-		entry.tag_type = tag
+		entry.tag_type = char_to_acltag(typechar)
 		if id is not None: entry.qualifier = id
 		entry.permset.read = perms >> 2
 		entry.permset.write = perms >> 1 & 1
