@@ -177,7 +177,8 @@ class FSAbilities:
 					log.Log("Warning: File system no longer needs quoting, "
 							"but will retain for backwards compatibility.", 2)
 					self.chars_to_quote = old_chars
-				else: log.Log.FatalError("""New quoting requirements
+				elif Globals.chars_to_quote is None:
+					log.Log.FatalError("""New quoting requirements
 
 This may be caused when you copy an rdiff-backup directory from a
 normal file system on to a windows one that cannot support the same
@@ -200,7 +201,9 @@ rdiff-backup-data/chars_to_quote.
 	def set_hardlinks(self, testdir):
 		"""Set self.hardlinks to true iff hard linked files can be made"""
 		hl_source = testdir.append("hardlinked_file1")
-		hl_dest = testdir.append("hardlinked_file2")
+		hl_dir = testdir.append("hl")
+		hl_dir.mkdir()
+		hl_dest = hl_dir.append("hardlinked_file2")
 		hl_source.touch()
 		try:
 			hl_dest.hardlink(hl_source.path)
